#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "VerteksCube.h"
#include "Globals.h"

subCube::subCube()
{
	for (int k=0; k<2; k++){
		for (int j=0; j<2; j++){
			for (int i=0; i<2; i++){
				cubes[i][j][k] = new Cube;
			}
		}
	}
}

Cube::Cube()
{
	//printf("Cube");
	skin = (float)0.05;
	cv = NULL;
}

Cube::Cube(cubeVertekses *corners)
{	
	skin = (float)0.05;
	material.r=(float)0.67; material.g=(float)0.26; material.b=(float)0.15;

	cv = corners;	//Cubes 27 vertices
	isLeaf = false;	//set to false first, so this variabel is false when split() is called. (See MultiRes() )
	isMRes = false;

	//Calling split method if the cube is larger than default resolution
	if ( abs(corners->v[0][0][0]->origPos.x - corners->v[2][0][0]->origPos.x) > 1.0 ){
		split();
	}
	else {
		isLeaf=true;
		AddMyselfToMyVertices();
		//RetVertN(cv->v[0][0][0]);//FIXME: does this call do anything???
	}
	//printf("incVertList lengde: %i\n", incVertList.size());
}

Cube::~Cube()
{
}

void Cube::split(){

	//myChildren is not alocated before...
	myChildren = new subCube;
	//printf("i split... ");
	float dimOrg = abs( cv->v[0][0][0]->origPos.x - cv->v[2][0][0]->origPos.x );
	float dimAct = abs( cv->v[0][0][0]->pos.x - cv->v[2][0][0]->pos.x );//also y and z to be acsactly accurate

	/* Setting the position of the 19 of the 27 that is not sat before split() is called.
	That is: The internal nodes.*/
	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				/*[0][0][0] is lower, furthest back, left */ 
				if ( i==1 || j==1 || k==1){ //meaning this is an internal Vertex(not one of the corners)
					cv->v[i][j][k]->origPos.x = cv->v[0][0][0]->origPos.x + float(i)*(dimOrg/2);
					cv->v[i][j][k]->origPos.y = cv->v[0][0][0]->origPos.y + float(j)*(dimOrg/2);
					cv->v[i][j][k]->origPos.z = cv->v[0][0][0]->origPos.z + float(k)*(dimOrg/2);

					cv->v[i][j][k]->pos.x = cv->v[0][0][0]->pos.x + float(i)*(dimAct/2);
					cv->v[i][j][k]->pos.y = cv->v[0][0][0]->pos.y + float(j)*(dimAct/2);
					cv->v[i][j][k]->pos.z = cv->v[0][0][0]->pos.z + float(k)*(dimAct/2);
					//if(isLeaf)printf("IncVertList, size: %i\n", incVertList.size() );
					int h;
					/*If the Verteks already exist we want it to be the same!*/
					for (h=0; h<incVertList.size(); h++){
						if ( cv->v[i][j][k]->IsEqual(incVertList[h]) ){
							cv->v[i][j][k] = incVertList[h];
							break;
						}
					}
					/*Than it wasn't in incVertList from before and we add it*/
					if( h >= incVertList.size() ){
						//printf("pushed incVertList. lengde: %i\n", incVertList.size());
						incVertList.push_back(cv->v[i][j][k]);
					}
				}
			}
		}
	}
	
	for (int n=0; n<2; n++){
		for (int m=0; m<2; m++){
			for (int l=0; l<2; l++){
				makeChild(l,m,n);
			}
		}
	}
	
	if(isLeaf){ //then this call is from MultiRes() and not from the constructor.
		for (int n=0; n<2; n++){
			for (int m=0; m<2; m++){
				for (int l=0; l<2; l++){
					myChildren->cubes[l][m][n]->isMRes=true;
				}
			}
		}
	}
	isLeaf = false; //Setting itself not to be a leaf
}//void Cube::split()

void Cube::makeChild(int i, int j, int k)
{
	subCube* mCC = myChildren;
	cubeVertekses* corners = new cubeVertekses;
	
	for (int n=0; n<2; n++){
		for (int m=0; m<2; m++){
			for (int l=0; l<2; l++){
				/*[0][0][0] is lower, furthest back, left */
				corners->v[l*2][m*2][n*2] = cv->v[l+i][m+j][n+k];
			}
		}
	}
	mCC->cubes[i][j][k] = new Cube(corners);
}//Cube::makeChild

void Cube::MultiRes(int l)
{
	
	//First we remove this cube from all it's vertices' myCubes,
	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				cv->v[i][j][k]->RemoveCube(this);
			}
		}
	}
	//then we call split to split the cube
	split();

	if (l > 1){
		for (int r=0; r<2; r++){
			for (int q=0; q<2; q++){
				for (int p=0; p<2; p++){
					myChildren->cubes[p][q][r]->MultiRes( l-1 );
				}
			}
		}
	}
}

void Cube::MakeNewVert(Verteks* v)
{
	//new verteks //change reference //add this cube to new verteks myCUbes
	Verteks* newV = new Verteks;
	newV->origPos.x = v->origPos.x; newV->origPos.y = v->origPos.y; newV->origPos.z = v->origPos.z;
	newV->pos.x = v->pos.x; newV->pos.y = v->pos.y; newV->pos.z = v->pos.z;
	newV->isCut = true;
	newV->myCubes.push_back(this);

	//Find the correct vertex and change the reference to point to the new one
	bool found = false;
	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				if (v == cv->v[i][j][k]){
					found = true;
                    cv->v[i][j][k] = newV;
					//incVertList.push_back(cv->v[i][j][k]);
				}
			}
		}
	}
	if(!found)printf("Cube::MakeNewVert: v NOT found in cv->v!!!");
	//Verteks will be added to vertekslist after next call to MakeVertList

}//void Cube::MakeNewVert(Verteks* v)

void Cube::AddMyselfToMyVertices()
{
	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				cv->v[i][j][k]->myCubes.push_back(this);
			}
		}
	}
}//Cube::AddMyselfToMyVertices()

void Cube::RemoveMeFromMyVertices()
{
	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				cv->v[i][j][k]->RemoveCube(this);
			}
		}
	}
}//Cube::RemoveMeFromMyVertices()

Verteks** Cube::RetVertN(Verteks* v)
{
	Verteks** neighb;
	neighb = new Verteks*[3];
	/*for (int t=0; t<3; t++){
		neighb[t] = new Verteks;*/

	for (int k=0; k<2; k++){
		for (int j=0; j<2; j++){
			for (int i=0; i<2; i++){
				if ( v == cv->v[2*i][2*j][2*k] ){//gives only 0 and 2 as indices
					neighb[0]=cv->v[-(i*2)+2][j*2]     [k*2];
					neighb[1]=cv->v[i*2]     [-(j*2)+2][k*2];
					neighb[2]=cv->v[i*2]     [j*2]     [-(k*2)+2];
					return neighb;
				}
			}
		}
	}
	printf("NEVER HERE!!!");
	return NULL;
}

Cube* Cube::GetHitCube(hduVector3Dd proxyPos)
{
	if ( !inCube(proxyPos) ){
		return NULL;
	}
	//Is in Qube!
	Cube* myRet;
	if (!isLeaf){
        for (int k=0; k<2; k++){
			for (int j=0; j<2; j++){
				for (int i=0; i<2; i++){
					myRet = myChildren->cubes[i][j][k]->GetHitCube(proxyPos);
					if (myRet != NULL){
						return myRet;
					}
				}
			}
		}
        return NULL;
	}
	else {
		return this;
	}
}

Verteks* Cube::GetNearestVert(hduVector3Dd proxyPos)
{
	if ( !inCube(proxyPos) ){
		return NULL;
	}
	
	//We have a HIT!! :-)
	Verteks* myRet;
	if (!isLeaf){
        for (int k=0; k<2; k++){
			for (int j=0; j<2; j++){
				for (int i=0; i<2; i++){
					myRet = myChildren->cubes[i][j][k]->GetNearestVert(proxyPos);
					if (myRet != NULL){
						return myRet;
					}
				}
			}
		}
		return NULL;
	}
	else {
        return GetNearestVertInThisCube(proxyPos);
	}
}

Verteks* Cube::GetNearestVertInThisCube(hduVector3Dd proxyPos)
{
	int minDistVert[3];
	double minDist = 1000;
	hduVector3Dd tmpVertice;

	for (int k=0; k<3; k+=2){
		for (int j=0; j<3; j+=2){
			for (int i=0; i<3; i+=2){
				tmpVertice[0]=cv->v[i][j][k]->pos.x; tmpVertice[1]=cv->v[i][j][k]->pos.y; tmpVertice[2]=cv->v[i][j][k]->pos.z;
				if (proxyPos.distance(tmpVertice) < minDist){
					minDist = proxyPos.distance(tmpVertice);
					minDistVert[0]=i; minDistVert[1]=j; minDistVert[2]=k;
				}
			}
		}
	}
	return cv->v[ minDistVert[0] ][ minDistVert[1] ][ minDistVert[2] ];
}

bool Cube::inCube(hduVector3Dd proxyPos)
{
	if (cv->v[0][0][0]->pos.x < proxyPos[0]+skin && cv->v[2][0][0]->pos.x > proxyPos[0]-skin &&
		cv->v[0][0][0]->pos.y < proxyPos[1]+skin &&	cv->v[0][2][0]->pos.y > proxyPos[1]-skin &&
		cv->v[0][0][0]->pos.z < proxyPos[2]+skin &&	cv->v[0][0][2]->pos.z > proxyPos[2]-skin){
		return true;
	}	
	else {
		return false;
	}
}


/******************** SCATCH *******************/

//Verteks* Cube::GetNearestVert(hduVector3Dd proxyPos)
//{
//	if (!isLeaf){
//		if ( inCube(proxyPos) ){
//            for (int k=0; k<2; k++){
//				for (int j=0; j<2; j++){
//					for (int i=0; i<2; i++){
//						myChildren->cubes[i][j][k]->GetNearestVert(proxyPos);
//					}
//				}
//			}
//		}
//		else{} //Do nothing, the proxyPos is not in this cube
//	}
//	else {
//		if ( inCube(proxyPos) ){
//			return NearestVertInThisCube(proxyPos);
//		}
//		else{} //Do nothing.
//	}
//}

/*for (int i=0; i<2; i++){
		for (int j=0; j<2; j++){
			for (int k=0; k<2; k++){
				printf("x,y,z-pos i cc: , %i %i %i: %.3f %.3f %.3f\n", i*2, j*2, k*2, cv->v[i*2][j*2][k*2].pos.x, cv->v[i*2][j*2][k*2].pos.y, cv->v[i*2][j*2][k*2].pos.z);
				
			}
		}
	}
	printf("\n");*/