#ifndef __MESH_H
#define __MESH_H

# include <GL/glut.h>
#include <math.h>
#include <iostream>

#include <HL/hl.h>
#include <HDU/hduMatrix.h>
#include <HDU/hduError.h>

#include "VerteksCube.h"

struct cube
{
	//four x,y,z coords for each face of the cube
	coord left[4];
	coord right[4];
	coord front[4];
	coord back[4];
	coord top[4];
	coord bottom[4];
};

struct cube_
{
	//four x,y,z coords for each face of the cube
	//i,j,k indices into the vertices-array (cubeCoords)
	int left[4][3];
	int right[4][3];
	int front[4][3];
	int back[4][3];
	int top[4][3];
	int bottom[4][3];
};

class Mesh
{
public:
	Mesh();
	~Mesh();

    void initGeometry(int dimCubes);
	void GenerateMesh();
	void GetTouchedCube(hduVector3Dd proxyPos, int *hitCubeIndices);
	void Old_Simulate(hduVector3Dd reactionForce, hduVector3Dd proxyPos);
	void GetTouchedFace(hduVector3Dd proxyPos, int hitCube[], int *hitFace);
	void UpdateGeometry();

	//NEW DESIGN!!
	void DrawCubes(Cube* rootNode);
	void DrawCube(Cube* leaf);
	void MakeVertList(Cube* rootNode);
	void VAddNeighbors();
	void Simulate();
	void RemoveFromVerteksList(Verteks* v);	//removes v from verteksList

	vector<Verteks*> verteksList;

	//TEST!!
	void CubeTest();
	void TestLines();
	void DrawCoordSys();

	//bool firstRP;

private:
	coord*** orgCubeC;	//3D array of the original cubecoordinates
	coord*** cubeCoords;//3D array of vertices for the cubes
	cube*** cubeGrid;	//3D array of cubes
	rgb*** cubeColor;	//3D array holding color for each cube

	int dimCubes;

	int prevHitFace;//for use in GetTouchedFace() only.

};// Class Mesh

#endif // __MESH_H_