/*****************************************************************************

Copyright (c) 2004 SensAble Technologies, Inc. All rights reserved.

OpenHaptics(TM) toolkit. The material embodied in this software and use of
this software is subject to the terms and conditions of the clickthrough
Development License Agreement.

For questions, comments or bug reports, go to forums at: 
    http://dsc.sensable.com

Module Name:

  SurgSim.cpp

Description: 

  This example demonstrates basic haptic rendering of a shape.

******************************************************************************/

#include <math.h>
#include <assert.h>

#if defined(WIN32)
#include <windows.h>
#endif

#if defined(WIN32) || defined(linux)
#include <GL/glut.h>
#elif defined(__APPLE__)
#include <GLUT/glut.h>
#endif

#include <HL/hl.h>
#include <HDU/hduMatrix.h>
#include <HDU/hduError.h>

#include <HLU/hlu.h>

/* TSH: include files*/
#include <iostream>
/* TSH: My own include files*/
#include "Mesh.h"
#include "Tool.h"
#include "DrawString.h"
#include "VerteksCube.h"
#include "Globals.h"

/* TSH: Instanses of the other classes*/
Mesh mesh;
Tool tool;
Cube* meshRoot;
//Global verteksList
vector<Verteks*> incVertList;

/* TSH variables*/
#define screenWidth 700
#define screenHeight 700
int gridDim = 4;//NB nr denne endres, endre i destructoren til mesh
bool touched = false;
int mrLev = 1;	//How many levelss of multi resolution

/* Haptic device and rendering context handles. */
static HHD ghHD = HD_INVALID_HANDLE;
static HHLRC ghHLRC = 0;

/* Shape id for shape we will render haptically. */
HLuint gSphereShapeId;

#define CURSOR_SIZE_PIXELS 20
static double gCursorScale;
static GLuint gCursorDisplayList = 0;

/* Function prototypes. */
void glutDisplay(void);
void glutReshape(int width, int height);
void glutIdle(void);   
void glutMenu(int); 

void exitHandler(void);

void initGL();
void initHL();
void initScene();
void drawSceneHaptics();
void drawSceneGraphics();
void drawCursor();
void updateWorkspace();

void HLCALLBACK OnTouchShapeCallback(HLenum event, HLuint object, HLenum thread, HLcache *cache, void *userdata);
void HLCALLBACK OnDragSurface(HLenum event, HLuint object, HLenum thread, HLcache *cache, void *userdata);

void DisplayInfo();
double DetermineFPS();
void initOctTreeMesh();

/*******************************************************************************
 Initializes GLUT for displaying a simple haptic scene.
*******************************************************************************/
int main(int argc, char *argv[])
{
    glutInit(&argc, argv);
    
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

    glutInitWindowSize(700, 700);
    glutCreateWindow("Surgery Simulator");

    // Set glut callback functions.
    glutDisplayFunc(glutDisplay);
    glutReshapeFunc(glutReshape);
    glutIdleFunc(glutIdle);
    
    glutCreateMenu(glutMenu);
    glutAddMenuEntry("Quit", 0);
    glutAttachMenu(GLUT_RIGHT_BUTTON);    
    
    // Provide a cleanup routine for handling application exit.
    atexit(exitHandler);

    initScene();

    glutMainLoop();

    return 0;
}

/*******************************************************************************
 GLUT callback for redrawing the view.
*******************************************************************************/
void glutDisplay()
{   
    drawSceneHaptics();

    drawSceneGraphics();

	DisplayInfo();

    glutSwapBuffers();
}

/*******************************************************************************
 GLUT callback for reshaping the window.  This is the main place where the 
 viewing and workspace transforms get initialized.
*******************************************************************************/
void glutReshape(int width, int height)
{
    static const double kPI = 3.1415926535897932384626433832795;
    static const double kFovY = 40;

    double nearDist, farDist, aspect;

    glViewport(0, 0, width, height);

    // Compute the viewing parameters based on a fixed fov and viewing
    // a canonical box centered at the origin.
	
	//TSH: Endret midlertidig...
    nearDist = 8 + (gridDim/2); /*1.0 / tan((kFovY / 2.0) * kPI / 180.0);*/
	farDist = nearDist + 2*gridDim + 10; /*nearDist + 2.0;*/
    aspect = (double) width / height;
	printf("near:%f, far:%f",nearDist, farDist);
   
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(kFovY, aspect, nearDist, farDist);
	//TSH: midlertidig...
	//glFrustum(-3.0, 3.0, -3.0, 3.0, 2.5, 14.0);

    // Place the camera down the Z axis looking at the origin.
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	/* gluLookAt(0, 0, nearDist + 1.0, = ser alt fra 1.0 til 1.0-(farDist-nearDist) */
    gluLookAt( 8, 10, nearDist + 8,
               0,  3, 0,
               0,  1, 0);
    
    updateWorkspace();
}

/*******************************************************************************
 GLUT callback for idle state.  Use this as an opportunity to request a redraw.
 Checks for HLAPI errors that have occurred since the last idle check.
*******************************************************************************/
void glutIdle()
{
    HLerror error;

    while (HL_ERROR(error = hlGetError()))
    {
        fprintf(stderr, "HL Error: %s\n", error.errorCode);
        
        if (error.errorCode == HL_DEVICE_ERROR)
        {
            hduPrintError(stderr, &error.errorInfo,
                "Error during haptic rendering\n");
        }
    }
	
	mesh.Simulate();

    glutPostRedisplay();
}

/******************************************************************************
 Popup menu handler.
******************************************************************************/
void glutMenu(int ID)
{
    switch(ID) {
        case 0:
            exit(0);
            break;
    }
}

/*******************************************************************************
 Initializes the scene.  Handles initializing both OpenGL and HL.
*******************************************************************************/
void initScene()
{
	initOctTreeMesh();
	//mesh.initGeometry(gridDim);

    initGL();
    initHL();
}

/*******************************************************************************
 Sets up general OpenGL rendering properties: lights, depth buffering, etc.
*******************************************************************************/
void initGL()
{
	//mesh.firstRP = true;//testVar for DrawCube...

	//TSH: slr av all lys ting forelpig... 
	static const GLfloat light_model_ambient[] = {0.3f, 0.3f, 0.3f, 1.0f};
	static const GLfloat light0_diffuse[] = {0.9f, 0.9f, 0.9f, 0.9f};
	static const GLfloat light0_direction[] = {0.0f, 0.4f, 1.0f, 0.0f};

	//Enable depth buffering for hidden surface removal.
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);

	glPolygonMode(GL_FRONT/*_AND_BACK*/, GL_FILL);

    // Cull back faces.
    glCullFace(GL_BACK);
    glEnable(GL_CULL_FACE);

    // Setup other misc features.
    glEnable(GL_LIGHTING);
    glEnable(GL_NORMALIZE);
    glShadeModel(GL_SMOOTH);
    
    // Setup lighting model.
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);    
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_model_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, light0_direction);
    glEnable(GL_LIGHT0);   
	
}

/*******************************************************************************
 Initialize the HDAPI.  This involves initing a device configuration, enabling
 forces, and scheduling a haptic thread callback for servicing the device.
*******************************************************************************/
void initHL()
{
    HDErrorInfo error;

    ghHD = hdInitDevice(HD_DEFAULT_DEVICE);
    if (HD_DEVICE_ERROR(error = hdGetError()))
    {
        hduPrintError(stderr, &error, "Failed to initialize haptic device");
        fprintf(stderr, "Press any key to exit");
        getchar();
        exit(-1);
    }
    
    ghHLRC = hlCreateContext(ghHD);
    hlMakeCurrent(ghHLRC);

    // Enable optimization of the viewing parameters when rendering geometry for OpenHaptics.
    hlEnable(HL_HAPTIC_CAMERA_VIEW);

    // Generate id's for the three shapes.
    gSphereShapeId = hlGenShapes(1);

    hlTouchableFace(HL_FRONT/*_AND_BACK*/);

	// Setup event callbacks.
    hlAddEventCallback(HL_EVENT_TOUCH, HL_OBJECT_ANY, HL_CLIENT_THREAD, &OnTouchShapeCallback, NULL);
	hlAddEventCallback(HL_EVENT_UNTOUCH, HL_OBJECT_ANY, HL_CLIENT_THREAD, &OnTouchShapeCallback, NULL);

	// set motion tolerance to 0 so we get motion event when
    // when not moving.
    hlEventd(HL_EVENT_MOTION_LINEAR_TOLERANCE, 0);
	hlAddEventCallback(HL_EVENT_MOTION, HL_OBJECT_ANY, HL_CLIENT_THREAD, &OnDragSurface, NULL);
}

/*******************************************************************************
 This handler is called when the application is exiting.  Deallocates any state 
 and cleans up.
*******************************************************************************/
void exitHandler()
{
    // Deallocate the sphere shape id we reserved in initHL.
    hlDeleteShapes(gSphereShapeId, 1);

    // Free up the haptic rendering context.
    hlMakeCurrent(NULL);
    if (ghHLRC != NULL)
    {
        hlDeleteContext(ghHLRC);
    }

    // Free up the haptic device.
    if (ghHD != HD_INVALID_HANDLE)
    {
        hdDisableDevice(ghHD);
    }
	// Remove event callbacks.
    hlRemoveEventCallback(HL_EVENT_TOUCH, gSphereShapeId, HL_CLIENT_THREAD, OnTouchShapeCallback);
}

/*******************************************************************************
 Use the current OpenGL viewing transforms to initialize a transform for the
 haptic device workspace so that it's properly mapped to world coordinates.
*******************************************************************************/
void updateWorkspace()
{
    GLdouble modelview[16];
    GLdouble projection[16];
    GLint viewport[4];

    glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
    glGetDoublev(GL_PROJECTION_MATRIX, projection);
    glGetIntegerv(GL_VIEWPORT, viewport);

    hlMatrixMode(HL_TOUCHWORKSPACE);
    hlLoadIdentity();
    
    // Fit haptic workspace to view volume.
    hluFitWorkspace(projection);

    // Compute cursor scale.
    gCursorScale = hluScreenToModelScale(modelview, projection, viewport);
    gCursorScale *= CURSOR_SIZE_PIXELS;
}

/*******************************************************************************
 The main routine for displaying the scene.  Gets the latest snapshot of state
 from the haptic thread and uses it to display a 3D cursor.
*******************************************************************************/
void drawSceneGraphics()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);           

    // Draw 3D cursor at haptic device position.
    drawCursor();

    /* Draw a sphere using OpenGL. */
    //glutSolidSphere(0.5, 32, 32);
	//mesh.CubeTest();
	//mesh.TestLines();
	
	/* Draw my autogenerated mesh*/
	//mesh.GenerateMesh();

	// To awoid the
	glPushMatrix();
	glBegin(GL_QUADS);
	mesh.DrawCubes(meshRoot);
	glEnd();
	glPopMatrix();

	//mesh.DrawCoordSys();
	//mesh.firstRP=false;

}

/*******************************************************************************
 The main routine for rendering scene haptics.
*******************************************************************************/
void drawSceneHaptics()
{
    // Start haptic frame.  (Must do this before rendering any haptic shapes.)
    hlBeginFrame();

    // Set material properties for the shapes to be drawn.
    hlMaterialf(HL_FRONT_AND_BACK, HL_STIFFNESS, 0.7f);
    hlMaterialf(HL_FRONT_AND_BACK, HL_DAMPING, 0.1f);
    hlMaterialf(HL_FRONT_AND_BACK, HL_STATIC_FRICTION, 0.9f); //0.2
    hlMaterialf(HL_FRONT_AND_BACK, HL_DYNAMIC_FRICTION, 0.9f); //0.3

	hlCheckEvents();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	hlHintb(HL_SHAPE_DYNAMIC_SURFACE_CHANGE, HL_TRUE);

    // Start a new haptic shape.  Use the feedback buffer to capture OpenGL 
    // geometry for haptic rendering.
    //hlBeginShape(HL_SHAPE_FEEDBACK_BUFFER, gSphereShapeId);
	hlBeginShape(HL_SHAPE_DEPTH_BUFFER, gSphereShapeId);

    // Use OpenGL commands to create geometry.
	/*glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	mesh.GenerateMesh();
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);*/
	
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glPushMatrix();
	glBegin(GL_QUADS);
	mesh.DrawCubes(meshRoot);
	glEnd();
	glPopMatrix();
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

    hlEndShape();// End the shape.

    hlEndFrame();// End the haptic frame.
}


/*******************************************************************************
 Draws a 3D cursor for the haptic device using the current local transform,
 the workspace to world transform and the screen coordinate scale.
*******************************************************************************/
void drawCursor()
{
    static const double kCursorRadius = 0.5;
    static const double kCursorHeight = 1.5;
    static const int kCursorTess = 15;
    HLdouble proxyxform[16];

    GLUquadricObj *qobj = 0;

    glPushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT | GL_LIGHTING_BIT);
    glPushMatrix();

    if (!gCursorDisplayList)
    {
        gCursorDisplayList = glGenLists(1);
        glNewList(gCursorDisplayList, GL_COMPILE);
        qobj = gluNewQuadric();
               
        gluCylinder(qobj, 0.0, kCursorRadius, kCursorHeight,
                    kCursorTess, kCursorTess);
        glTranslated(0.0, 0.0, kCursorHeight);
        gluCylinder(qobj, kCursorRadius, 0.0, kCursorHeight / 5.0,
                    kCursorTess, kCursorTess);
    
        gluDeleteQuadric(qobj);
        glEndList();
    }
    
    // Get the proxy transform in world coordinates.
    hlGetDoublev(HL_PROXY_TRANSFORM, proxyxform);
    glMultMatrixd(proxyxform);

    // Apply the local cursor scale factor.
    glScaled(gCursorScale, gCursorScale, gCursorScale);

    glEnable(GL_COLOR_MATERIAL);
    glColor3f(0.0, 0.5, 1.0);

    glCallList(gCursorDisplayList);

    glPopMatrix(); 
    glPopAttrib();
}

void HLCALLBACK OnTouchShapeCallback(HLenum event, HLuint object, HLenum thread, HLcache *cache, void *userdata)
{
	if(event == HL_EVENT_TOUCH){
		touched = true;

        hduVector3Dd proxyPos;
		//std::cout << "Touched ObjectID: " << object << " at point ";
		hlCacheGetDoublev(cache, HL_PROXY_POSITION, proxyPos);
		//std::cout << proxyPos << "\n";
		hduVector3Dd reactionForce;
		hlCacheGetDoublev(cache, HL_DEVICE_FORCE, reactionForce);

		HLboolean buttDown;
		hlCacheGetBooleanv(cache, HL_BUTTON1_STATE, &buttDown);

		/* Apply Multi resolution */
		Cube* c = meshRoot->GetHitCube(proxyPos);
		if ( c ){
			if (!c->isMRes){
				c->MultiRes( mrLev );
				mesh.verteksList.clear();
				mesh.MakeVertList(meshRoot);
				mesh.VAddNeighbors();
			}
			/* Apply the forces on the nearest Verteks of the hit */
			//Verteks* v = meshRoot->GetNearestVert(proxyPos);
			Verteks* v = c->GetNearestVert(proxyPos);//we already have the hit cube in c
			if ( v ){
				v->ApplyExtForce(-(hduVector3Df)reactionForce);
			}
		}
		else {//Try again
            Verteks* v = meshRoot->GetNearestVert(proxyPos);
			if ( v ){
				v->ApplyExtForce(-(hduVector3Df)reactionForce);
			}
		}

	}
	else if(event == HL_EVENT_UNTOUCH){
		touched = false;
	}
}

void HLCALLBACK OnDragSurface(HLenum event, HLuint object, HLenum thread, HLcache *cache, void *userdata)
{
	if(touched){
		//std::cout << "Dragged On ObjectID: " << object << " at pos: ";
		hduVector3Dd reactionForce;
		hlCacheGetDoublev(cache, HL_DEVICE_FORCE, reactionForce);
		
		hduVector3Dd proxyPos;
		hlCacheGetDoublev(cache, HL_PROXY_POSITION, proxyPos);

		HLboolean buttDown;
		hlCacheGetBooleanv(cache, HL_BUTTON1_STATE, &buttDown);
		//std::cout << "button 1 " << (buttDown ? "down" : "up") << std::endl;
		
		int dummy = 0;
		Cube* c = meshRoot->GetHitCube(proxyPos);
		/*if (!c){
			dummy++;
		}*/
		if ( c ){
			//printf("dragHit!!");
			//if ( buttDown ) printf("button down");
			if (!c->isMRes){ /* Apply Multi resolution */
				c->MultiRes( mrLev );
				mesh.verteksList.clear();
				mesh.MakeVertList(meshRoot);
				printf("\nlengde verteksList: %i \n", mesh.verteksList.size() );
				mesh.VAddNeighbors();
			}
			/* Apply the forces on the nearest Verteks of the hit */
			Verteks* v = c->GetNearestVert(proxyPos);//we already have the hit cube in c
			if ( v ){
				if ( buttDown && !v->isCut ){ /* Apply cutting */
					//c->material.r=0; c->material.g=0.7; c->material.b=0.2;
					v->cut();
					mesh.verteksList.clear();
					mesh.MakeVertList(meshRoot);
					mesh.VAddNeighbors();
				}
				else{ /*Apply forces*/
                    v->ApplyExtForce(-(hduVector3Df)reactionForce);
				}
			}
		}
		else {//Try finding nearest vert again (from root)
            Verteks* v = meshRoot->GetNearestVert(proxyPos);
			if ( v ){
				v->ApplyExtForce(-(hduVector3Df)reactionForce);
			}
		}
	}//if touched
}//OnDragSurface(...)

void DisplayInfo()
{
	glDisable(GL_DEPTH_TEST);

    glPushMatrix();
    glLoadIdentity();

	// switch to 2d orthographic mMode for drawing text
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, screenWidth, screenHeight, 0);
	glMatrixMode(GL_MODELVIEW);
	
	glColor3f(1.0, 1.0, 1.0);

	DrawString(5, 20, GLUT_BITMAP_9_BY_15, "Framerate:     %f", DetermineFPS());
	//DrawString(5, 40, GLUT_BITMAP_9_BY_15, "Timestep:      %3.3f", dt);
	//printf("Framerate: %f\n", DetermineFPS());
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
	
	// turn depth and lighting back on for 3D rendering
	glEnable(GL_DEPTH_TEST);
}

double DetermineFPS(void)
{
	static unsigned int	lastMs = glutGet(GLUT_ELAPSED_TIME);
	unsigned int		currentMs = glutGet(GLUT_ELAPSED_TIME);
	unsigned int		framesMs = currentMs - lastMs;
	static unsigned int	framesCounted = 0;
	static double fps = 0;
	
	framesCounted++;

	#define DURATION_TO_TIME 1000	// duration in miliseconds for calculating fps

	if(framesMs >= DURATION_TO_TIME)
	{
		fps = (double)1000 * framesCounted / framesMs;
		//printf("last: %ld  curr: %ld\n", g.lastClock, currentClock);
		//printf("clocks: %d  cps: %d  FPS: %f\n", clock(), CLOCKS_PER_SEC, fps);
		
		framesCounted = 0;
		//g.framesTime = 0;

		lastMs = currentMs;
	}
	return fps;
}

void initOctTreeMesh()
{

	cubeVertekses* startVertices = new cubeVertekses;
	/*NB: Must be of 2^n*/
	int dim = 4;

	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				if ( i==1 || j==1 || k==1){ //meaning this is an internal Vertex(not one of the corners)
				}
				else {
					startVertices->v[i][j][k]->origPos.x = startVertices->v[i][j][k]->pos.x = -(dim/2)+ i*(dim/2);
					startVertices->v[i][j][k]->origPos.y = startVertices->v[i][j][k]->pos.y = -(dim/2)+ j*(dim/2);
					startVertices->v[i][j][k]->origPos.z = startVertices->v[i][j][k]->pos.z = -(dim/2)+ k*(dim/2);
					incVertList.push_back(startVertices->v[i][j][k]);
				}
			}
		}
	}
	meshRoot = new Cube(startVertices);
	mesh.MakeVertList(meshRoot);
	mesh.VAddNeighbors();

	printf("\nlengde verteksList: %i \n", mesh.verteksList.size() );
	printf("lengde incVertList: %i \n", incVertList.size() );

	/*for (int i=0; i<mesh.verteksList.size(); i++){
		printf("nr of neighbors in %i: %i\n", i, mesh.verteksList[i]->vNeig.size());
	}*/

	/*hduVector3Dd t_0;
	t_0.set(1,2,3);
	t_0 = -t_0;
	hduVector3Dd t_1;
	t_1.set(1,2,2);
	std::cout << "vec: " << t_0 <<"\n";*/

	/*vector<int> test;
	test.push_back(2); test.push_back(3); test.push_back(5);
	std::cout <<"size: "<< test.size() << test[0] << test[1] << test[2] << "\n";
	test.erase(test.begin() + 1);
	std::cout <<"size: "<< test.size() << test[0] << test[1] << <<"\n";*/
	
}

/******************************************************************************/

	/*for( int i=0; i<mesh.verteksList.size(); i++){
		if( mesh.verteksList[i]->origPos.x == -1 && mesh.verteksList[i]->origPos.y == 1 && mesh.verteksList[i]->origPos.z == 2){
			for (int j=0; j<mesh.verteksList[i]->vNeig.size(); j++){
				cout << "nabo " << j << ": ";
				mesh.verteksList[i]->vNeig[j]->printOrigPos();
			}
		}
	}*/

	/*printf("\nlengde verteksList: %i \n", mesh.verteksList.size() );
	printf("lengde incVertList: %i \n", incVertList.size() );*/


	/*for( int i=0; i<mesh.verteksList.size(); i++){
		if( mesh.verteksList[i]->origPos.x == -1 && mesh.verteksList[i]->origPos.y == 1.5 && mesh.verteksList[i]->origPos.z == 2){
			for (int j=0; j<mesh.verteksList[i]->vNeig.size(); j++){
				cout << "nabo " << j << ": ";
				mesh.verteksList[i]->vNeig[j]->printOrigPos();
			}
		}
	}*/

		/*hduVector3Dd devPos;
		hdGetDoublev(HD_CURRENT_POSITION, devPos);*/
		//std::cout << proxyPos << devPos << "\n";

		//mesh.Old_Simulate(reactionForce, proxyPos);

//Cube* hitCube = meshRoot->GetHitCube(proxyPos);
		//if (hitCube){
		//  /*hitCube->material.r=0;
		//	hitCube->material.g=0;
		//	hitCube->material.b=1;*/
		//	/*hitCube->isLeaf=false;
		//	hitCube->isLeaf=true;*/
		//}
		//else std::cout << "HIT OUTSIDE SKIN (" << proxyPos << ")!!!\n";

//printf("StartPos: %f\n", startVertices->v[1][0][0].origPos.x);

	/*printf("x-y-z i kube: %f %f %f\n",	meshRoot->myChildren->cubes[0][0][1]->cv->v[0][0][0]->vNeig[0]->origPos.x,
										meshRoot->myChildren->cubes[0][0][1]->cv->v[0][0][0]->vNeig[0]->origPos.y,
										meshRoot->myChildren->cubes[0][0][1]->cv->v[0][0][0]->vNeig[0]->origPos.z );*/
	//printf("lengde: %i\n", startVertices->v[1][1][1]->myCubes.size());
	//printf("ETTER HERP Pos: %f\n", startVertices->v[1][0][0].origPos.x);

	/*for (int i=0; i<mesh.verteksList.size(); i++){
		printf("pos of verteks %i: %f %f %f\n", i,	mesh.verteksList[i]->origPos.x,
													mesh.verteksList[i]->origPos.y,
													mesh.verteksList[i]->origPos.z);
	}
	printf("pos: %f %f %f\n",	meshRoot->myChildren->cubes[0][0][0]->cv->v[0][0][0]->pos.x,
								meshRoot->myChildren->cubes[0][0][0]->cv->v[0][0][0]->pos.y,
								meshRoot->myChildren->cubes[0][0][0]->cv->v[0][0][0]->pos.z);*/
