#include <stdio.h>
#include <stdlib.h>

#include "VerteksCube.h"

cubeVertekses::cubeVertekses()
{
	for (int k=0; k<3; k++){
		for (int j=0; j<3; j++){
			for (int i=0; i<3; i++){
				v[i][j][k] = new Verteks;
			}
		}
	}
}

Verteks::Verteks()
{
	//printf("\nVERTEX_DEFAULT!!\n");
	isCut = false;
}

Verteks::Verteks(coord oPos)
{
	origPos = oPos;
    printf("\nVERTEX_COORD!!\n");
	printf("myPos:  %f %f %f\n", origPos.x, origPos.y, origPos.z);	
	isCut = false;
}

Verteks::~Verteks()
{
}

void Verteks::cut()
{
	for (int i=0; i<myCubes.size(); i++){
		myCubes[i]->MakeNewVert(this);
	}
	//NB: Clear verteksList must always be called after cut is called
	//printf("Verteks::void cut(): cut finished.");

}//Verteks::cut()

void Verteks::addNeighb()
{
	//each call to a cubes RetVertN() returns all three neigbours of the calling Verteks in this cube
	Verteks** tmp;

	for(int i=0; i<myCubes.size(); i++){
		
		tmp = myCubes[i]->RetVertN(this);
		for(int j=0; j<3; j++){
			vNeig.push_back(tmp[j]);
			for (int k=0; k<(vNeig.size()-1); k++){
				/*If the vertex is found in the list before the last element, it's(the one thats just added) popped off again*/
				if ( vNeig[k] == tmp[j] ){
					vNeig.pop_back();
					break;
				}
			}
		}
	}
}//void Verteks::addNeighb()

void Verteks::RemoveCube(Cube* c)
{
	int n = myCubes.size();
	for (int i=0; i<n; i++) {
		if( myCubes[i] == c){
			myCubes.erase(myCubes.begin()+i); //gets the (i+1)'th element. THAT IS myCubes[i]!!
		}
		break;
	}
	if (i >= n) printf("Verteks::RemoveCube(): cube not found!!!");
}

void Verteks::ApplyExtForce(hduVector3Df force)
{
    ext_force = force;
}

void Verteks::ClearExtForce()
{
	ext_force.set(0.0,0.0,0.0);
}

void Verteks::ReduceExtForce(float alfa)
{
	ext_force = ext_force - ext_force*alfa;
}

hduVector3Df Verteks::GetNailForce()
{
	hduVector3Df nailForce;

	if(!isCut){
		hduVector3Df myPos;
		myPos.set(pos.x, pos.y, pos.z);
		hduVector3Df myOrigPos;
		myOrigPos.set(origPos.x, origPos.y, origPos.z);
	
		nailForce = myOrigPos - myPos;
		return nailForce;
	}
	else {
		nailForce.set(0,0,0);
		return nailForce;
	}
}

hduVector3Df Verteks::SumNeigForce()
{
	/*Letting the restlength of the structural springs be less then the original distance,
	  "initiates the mesh with tension"*/
	float tensFac = (float)0.95;

	/*prefix v for vector and p for point*/
	hduVector3Df vIntForce;	//intern forces
	vIntForce.set(0,0,0);
	hduVector3Df vTmpForce;
	vTmpForce.set(0,0,0);
	
	//hduVector3Df
	float origDist;
	float curDist;
	//Org for origPos and Cur for current pos (pos)
	hduVector3Df pMeeOrg; //this coord
	hduVector3Df pMeeCur;
	pMeeOrg.set(origPos.x, origPos.y, origPos.z);
	pMeeCur.set(pos.x, pos.y, pos.z);

	hduVector3Df pCurNeigOrg; //current neighbor. Set inside the for-loop
	hduVector3Df pCurNeigCur;
	
	for (int i=0; i<vNeig.size(); i++){
		pCurNeigOrg.set(vNeig[i]->origPos.x, vNeig[i]->origPos.y, vNeig[i]->origPos.z);
		pCurNeigCur.set(vNeig[i]->pos.x, vNeig[i]->pos.y, vNeig[i]->pos.z);
		origDist = pMeeOrg.distance(pCurNeigOrg);
		curDist = pMeeCur.distance(pCurNeigCur);
	
		// neigb - mee: force on mee: towards neighb 
		//vTmpForce = (pMeeCur - pCurNeigCur) *  ( (curDist - origDist)/curDist );
		
		if( curDist > origDist*tensFac ){ //forces against neighbor
			vTmpForce = pCurNeigCur - pMeeCur;
		}
		else if ( curDist < origDist*tensFac ){ //forces away from neighbor
			vTmpForce = pMeeCur - pCurNeigCur;
		}
		else vTmpForce.set(0,0,0);

        vIntForce += vTmpForce;
	}
	return vIntForce * 0.5;
}

void Verteks::Move()/*float alfa, float k_x*/
{
	float extFFac = 1.0;

	hduVector3Df neighb = SumNeigForce();	//Force from neighbors
	hduVector3Df nail = GetNailForce();		//Force from origo
	
	ext_force *= extFFac;
	hduVector3Df tmpF = ext_force + neighb + nail;

	//if (origPos.x == 0 && origPos.y == 0 && origPos.z == 2){
	//	printf("neig, nail, ext: %f %f %f\n", neighb[2], nail[2], ext_force[2] );
	//	//printf("nail:   %f %f %f\n",  nail[0], nail[1], nail[2] );
	//	//printf("ext:    %f %f %f\n",  ext_force[0], ext_force[1], ext_force[2] );
	//	//printPos();
	//}

	pos.x += (float)tmpF[0] * alfa;
	pos.y += (float)tmpF[1] * alfa;
	pos.z += (float)tmpF[2] * alfa;
}

bool Verteks::IsEqual(Verteks* v)
{
	if (v->origPos.x == this->origPos.x &&
		v->origPos.y == this->origPos.y &&
		v->origPos.z == this->origPos.z   ){
		return true;
	}
	else return false;
}

void Verteks::printPos()
{
	printf("myPos: %f %f %f\n", pos.x, pos.y, pos.z);
}

void Verteks::printOrigPos()
{
	printf("myOrigPos: %f %f %f\n", origPos.x, origPos.y, origPos.z);
}

//######################## SCATCH ###########################

//printf("\n neighb: %f %f %f",  neighb[0], neighb[1], neighb[2] );
//printf("\n ext: %f %f %f\n",  ext_force[0], ext_force[1], ext_force[2] );
