#ifndef __VERTEKS_CUBE_H
#define __VERTEKS_CUBE_H

#include <vector>

#include <HL/hl.h>
#include <HDU/hduMatrix.h>
#include <HDU/hduError.h>

using namespace std;

class Cube;
class Verteks;

struct cubeVertekses;
struct subCube;
struct coord;
struct rgb;

//Used to build the list of vertekses incrementally as the cubes are made
//static vector<Verteks*> incVertList;
extern vector<Verteks*> incVertList;

//static float k_x = 10.0;	//spring stiffness
static float alfa = 0.002;	//stepsize


struct coord
{
	float x;
	float y;
	float z;
};

struct rgb
{
	float r;
	float g;
	float b;
};

struct subCube{ //struct for holding a cubes children
	subCube();

	Cube* cubes[2][2][2];
};

struct cubeVertekses{ //struct for holding a cubes vertices
	cubeVertekses();

	Verteks* v[3][3][3];
};

class Verteks
{
public:
	Verteks();
	Verteks(coord origPos);//not in use
	~Verteks();

	coord origPos;	//original position
	coord pos;		//actual position
	vector<Cube*> myCubes;	//All the cubes that have this vertex as one of theirs
	vector<Verteks*> vNeig;	//A verteks' neighbors
	hduVector3Df ext_force;	//Force applied by the user
	bool isCut;

	void addNeighb();
	void ApplyExtForce(hduVector3Df force);	// Only sets ext_force to force
	void Move();				/*float alfa, float k_x*/
	void ClearExtForce();		//Sets ext_force to [0,0,0]
	bool IsEqual(Verteks* v);	//Returns true if v has the same origPos as this, else false
	void RemoveCube(Cube* c);	//Called from the verteks' cube it's split in MR
	void cut();					//
	void printPos();
	void printOrigPos();

private:
	
	void ReduceExtForce(float alfa); //Not in use
	hduVector3Df SumNeigForce();	//returns the sum of the forces from all the neighbor vertekses
	hduVector3Df GetNailForce();	//Force from origPos

};//class Verteks

class Cube
{
public:
	Cube();
	Cube(cubeVertekses *corners);
	~Cube();

	float skin;
	rgb material;
	bool isMRes;		// Sat to false in constructor. Sat to true if a cube is plit to Multi resolution.
	bool isLeaf;		// Sat to true in constructor and to false when split is called.
	cubeVertekses* cv;	// 3D-array of all the 27 vertices of the cube. Only the 8 corners initialized in leaf-nodes.
	subCube* myChildren;// A struct with a [2][2][2]-array of cubes. All children of this cube.

	Verteks** RetVertN(Verteks* v);					//returning the neighbors of v in this cube
	Verteks* GetNearestVert(hduVector3Dd proxyPos);	//
	Cube* GetHitCube(hduVector3Dd proxyPos);		//Like GetNearestVert, only returning the hit cube instead of the nearest v in it
	bool inCube(hduVector3Dd proxyPos);	//help method for GetNearestVert() and GetHitCube()
	void MultiRes(int levels);			//called to split the cube (levels tels in how many levels to split)
	void MakeNewVert(Verteks* v);		//called from a Verteks thats being cut. Letting all the cubes that this Verteks
										//constitutes, make their own instance of this Verteks actually implements the cut.

private:
	
	void split(); //split the Cube in 8 child-cubes
	void makeChild(int i, int j, int k);
	void AddMyselfToMyVertices();	//every leaf-cube ads itself to its vertices' myCubes.
	void RemoveMeFromMyVertices();	//Calls a function in all its vertices that remove this cube from its myCubes
									//Done when this cube is split in MR
	Verteks* GetNearestVertInThisCube(hduVector3Dd proxyPos);	//help method for GetNearestVert()

};//class Cube

#endif // __VERTEKS_CUBE_H